@@ -24,10 +24,12 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <string.h>
+#include <limits.h>
 #include <signal.h>
 #include <time.h>
 #include <locale.h>
 #include <assert.h>
+#include <time.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/ipc.h>
@@ -43,6 +45,7 @@
 #include "cgroup.h"
 #include "profile.h"
 #include "lib/rand.h"
+#include "memalign.h"
 
 unsigned long page_mask;
 unsigned long page_size;
@@ -62,8 +65,8 @@ static struct fio_mutex *startup_mutex;
 static struct fio_mutex *writeout_mutex;
 static volatile int fio_abort;
 static int exit_value;
-static struct itimerval itimer;
 static pthread_t gtod_thread;
+static pthread_t disk_util_thread;
 static struct flist_head *cgroup_list;
 static char *cgroup_mnt;
 
@@ -100,33 +103,17 @@ static void terminate_threads(int group_id)
 			 * if the thread is running, just let it exit
 			 */
 			if (td->runstate < TD_RUNNING)
-				kill(td->pid, SIGQUIT);
+				kill(td->pid, SIGTERM);
 			else {
 				struct ioengine_ops *ops = td->io_ops;
 
-				if (ops && (ops->flags & FIO_SIGQUIT))
-					kill(td->pid, SIGQUIT);
+				if (ops && (ops->flags & FIO_SIGTERM))
+					kill(td->pid, SIGTERM);
 			}
 		}
 	}
 }
 
-static void status_timer_arm(void)
-{
-	itimer.it_value.tv_sec = 0;
-	itimer.it_value.tv_usec = DISK_UTIL_MSEC * 1000;
-	setitimer(ITIMER_REAL, &itimer, NULL);
-}
-
-static void sig_alrm(int fio_unused sig)
-{
-	if (threads) {
-		update_io_ticks();
-		print_thread_status();
-		status_timer_arm();
-	}
-}
-
 /*
  * Happens on thread runs with ctrl-c, ignore our own SIGQUIT
  */
@@ -143,15 +130,47 @@ static void sig_int(int sig)
 	}
 }
 
+static void *disk_thread_main(void *data)
+{
+	fio_mutex_up(startup_mutex);
+
+	while (threads) {
+		usleep(DISK_UTIL_MSEC * 1000);
+		if (!threads)
+			break;
+		update_io_ticks();
+		print_thread_status();
+	}
+
+	return NULL;
+}
+
+static int create_disk_util_thread(void)
+{
+	int ret;
+
+	ret = pthread_create(&disk_util_thread, NULL, disk_thread_main, NULL);
+	if (ret) {
+		log_err("Can't create disk util thread: %s\n", strerror(ret));
+		return 1;
+	}
+
+	ret = pthread_detach(disk_util_thread);
+	if (ret) {
+		log_err("Can't detatch disk util thread: %s\n", strerror(ret));
+		return 1;
+	}
+
+	dprint(FD_MUTEX, "wait on startup_mutex\n");
+	fio_mutex_down(startup_mutex);
+	dprint(FD_MUTEX, "done waiting on startup_mutex\n");
+	return 0;
+}
+
 static void set_sig_handlers(void)
 {
 	struct sigaction act;
 
-	memset(&act, 0, sizeof(act));
-	act.sa_handler = sig_alrm;
-	act.sa_flags = SA_RESTART;
-	sigaction(SIGALRM, &act, NULL);
-
 	memset(&act, 0, sizeof(act));
 	act.sa_handler = sig_int;
 	act.sa_flags = SA_RESTART;
@@ -160,7 +179,7 @@ static void set_sig_handlers(void)
 	memset(&act, 0, sizeof(act));
 	act.sa_handler = sig_quit;
 	act.sa_flags = SA_RESTART;
-	sigaction(SIGQUIT, &act, NULL);
+	sigaction(SIGTERM, &act, NULL);
 }
 
 /*
@@ -479,7 +498,6 @@ static void do_verify(struct thread_data *td)
 				clear_io_u(td, io_u);
 			} else if (io_u->resid) {
 				int bytes = io_u->xfer_buflen - io_u->resid;
-				struct fio_file *f = io_u->file;
 
 				/*
 				 * zero read, fail
@@ -497,6 +515,7 @@ static void do_verify(struct thread_data *td)
 				if (ddir_rw(io_u->ddir))
 					td->ts.short_io_u[io_u->ddir]++;
 
+				f = io_u->file;
 				if (io_u->offset == f->real_file_size)
 					goto sync_done;
 
@@ -778,7 +797,7 @@ static void cleanup_io_u(struct thread_data *td)
 		io_u = flist_entry(entry, struct io_u, list);
 
 		flist_del(&io_u->list);
-		free(io_u);
+		fio_memfree(io_u, sizeof(*io_u));
 	}
 
 	free_io_mem(td);
@@ -825,8 +844,9 @@ static int init_io_u(struct thread_data *td)
 		if (td->terminate)
 			return 1;
 
-		if (posix_memalign(&ptr, cl_align, sizeof(*io_u))) {
-			log_err("fio: posix_memalign=%s\n", strerror(errno));
+		ptr = fio_memalign(cl_align, sizeof(*io_u));
+		if (!ptr) {
+			log_err("fio: unable to allocate aligned memory\n");
 			break;
 		}
 
@@ -846,7 +866,7 @@ static int init_io_u(struct thread_data *td)
 				 * Fill the buffer with the pattern if we are
 				 * going to be doing writes.
 				 */
-				fill_pattern(td, io_u->buf, max_bs, io_u);
+				fill_pattern(td, io_u->buf, max_bs, io_u, 0, 0);
 			}
 		}
 
@@ -970,8 +990,10 @@ void reset_all_stats(struct thread_data *td)
 		td->io_issues[i] = 0;
 		td->ts.total_io_u[i] = 0;
 	}
-	
+
 	fio_gettime(&tv, NULL);
+	td->ts.runtime[0] = 0;
+	td->ts.runtime[1] = 0;
 	memcpy(&td->epoch, &tv, sizeof(tv));
 	memcpy(&td->start, &tv, sizeof(tv));
 }
@@ -1010,7 +1032,7 @@ static int exec_string(const char *string)
  */
 static void *thread_main(void *data)
 {
-	unsigned long long runtime[2], elapsed;
+	unsigned long long elapsed;
 	struct thread_data *td = data;
 	pthread_condattr_t attr;
 	int clear_state;
@@ -1049,14 +1071,18 @@ static void *thread_main(void *data)
 	 */
 	fio_mutex_remove(td->mutex);
 
-	if (td->o.uid != -1U && setuid(td->o.uid)) {
-		td_verror(td, errno, "setuid");
-		goto err;
-	}
+	/*
+	 * A new gid requires privilege, so we need to do this before setting
+	 * the uid.
+	 */
 	if (td->o.gid != -1U && setgid(td->o.gid)) {
 		td_verror(td, errno, "setgid");
 		goto err;
 	}
+	if (td->o.uid != -1U && setuid(td->o.uid)) {
+		td_verror(td, errno, "setuid");
+		goto err;
+	}
 
 	/*
 	 * May alter parameters that init_io_u() will use, so we need to
@@ -1128,7 +1154,6 @@ static void *thread_main(void *data)
 	fio_gettime(&td->epoch, NULL);
 	getrusage(RUSAGE_SELF, &td->ts.ru_start);
 
-	runtime[0] = runtime[1] = 0;
 	clear_state = 0;
 	while (keep_running(td)) {
 		fio_gettime(&td->start, NULL);
@@ -1153,11 +1178,11 @@ static void *thread_main(void *data)
 
 		if (td_read(td) && td->io_bytes[DDIR_READ]) {
 			elapsed = utime_since_now(&td->start);
-			runtime[DDIR_READ] += elapsed;
+			td->ts.runtime[DDIR_READ] += elapsed;
 		}
 		if (td_write(td) && td->io_bytes[DDIR_WRITE]) {
 			elapsed = utime_since_now(&td->start);
-			runtime[DDIR_WRITE] += elapsed;
+			td->ts.runtime[DDIR_WRITE] += elapsed;
 		}
 
 		if (td->error || td->terminate)
@@ -1174,15 +1199,15 @@ static void *thread_main(void *data)
 
 		do_verify(td);
 
-		runtime[DDIR_READ] += utime_since_now(&td->start);
+		td->ts.runtime[DDIR_READ] += utime_since_now(&td->start);
 
 		if (td->error || td->terminate)
 			break;
 	}
 
 	update_rusage_stat(td);
-	td->ts.runtime[0] = (runtime[0] + 999) / 1000;
-	td->ts.runtime[1] = (runtime[1] + 999) / 1000;
+	td->ts.runtime[0] = (td->ts.runtime[0] + 999) / 1000;
+	td->ts.runtime[1] = (td->ts.runtime[1] + 999) / 1000;
 	td->ts.total_run_time = mtime_since_now(&td->epoch);
 	td->ts.io_bytes[0] = td->io_bytes[0];
 	td->ts.io_bytes[1] = td->io_bytes[1];
@@ -1334,7 +1359,7 @@ static void reap_threads(int *nr_running, int *t_rate, int *m_rate)
 			if (WIFSIGNALED(status)) {
 				int sig = WTERMSIG(status);
 
-				if (sig != SIGQUIT)
+				if (sig != SIGTERM)
 					log_err("fio: pid=%d, got signal=%d\n",
 							(int) td->pid, sig);
 				td_set_runstate(td, TD_REAPED);
@@ -1474,14 +1499,14 @@ static void run_threads(void)
 			todo--;
 		} else {
 			struct fio_file *f;
-			unsigned int i;
+			unsigned int j;
 
 			/*
 			 * for sharing to work, each job must always open
 			 * its own files. so close them, if we opened them
 			 * for creation
 			 */
-			for_each_file(td, f, i) {
+			for_each_file(td, f, j) {
 				if (fio_file_open(f))
 					td_io_close_file(td, f);
 			}
@@ -1685,11 +1710,14 @@ int main(int argc, char *argv[])
 	}
 
 	startup_mutex = fio_mutex_init(0);
+	if (startup_mutex == NULL)
+		return 1;
 	writeout_mutex = fio_mutex_init(1);
+	if (writeout_mutex == NULL)
+		return 1;
 
 	set_genesis_time();
-
-	status_timer_arm();
+	create_disk_util_thread();
 
 	cgroup_list = smalloc(sizeof(*cgroup_list));
 	INIT_FLIST_HEAD(cgroup_list);
